/*******************************************************************
*  include files
*******************************************************************/
#include "device.h"
#include "stdio.h"
#include "math.h"


/*******************************************************************
*  global defines
*******************************************************************/
//==================================================
//	Modify as per requirement
//==================================================

// 	Number of ADCC frames to be received. This will determine number of ADC samples to receive and hence size of ADC buffer.
//	Each ADCC frame consists of 24 ADC samples, as we have enabled 24 ADCC events. So, total number of ADC samples  = 24 * BUFF_SIZE
#define 	BUFF_SIZE	2900		

//	ADC channel selection bits
#define     ADC_SEL     1			//	0: ADC0					1: ADC1
#define     ADC_CHNL    0			//	ADC channel ID: 0 to 7

//	selection of internal reference or external reference
#define 	EXT_REF		1			//	0: Internal refernce 	1: External refernce


//==================================================
//	Do Not Modify
//==================================================
#define 	BITM_ADCC_EVCTL_CHN_ID		0x000000F0
#define 	BITP_ADCC_EVCTL_CHN_ID		4

#define 	BLCKMAN_WINDOW_COEFF_SECT	24
#define 	COS_COEFF_SECT				28
#define 	SIN_COEFF_SECT				30

#define 	SPI_FLASH_BLOCK_SIZE		0x10000
#define		SPI_FLASH_BLOCK_COUNT		32


/*******************************************************************
*  User data type defines
*******************************************************************/

typedef struct
{
	float re;
	float im;
} complex_float_var;

typedef struct
{
  float SNR;
  float SNDR;
  float ENOB;
  float THD; 
}ADC_Perfomance_Metrices;


/*******************************************************************
* extern data variables
*******************************************************************/

extern unsigned short ADC_Data[];
extern ADC_Perfomance_Metrices ADC_para; 
extern unsigned char ADCC_TMR0_DONE, ADCC_TMR1_DONE; 


/*******************************************************************
*  function prototypes
*******************************************************************/

//	Initialisation functions
void SystemInit (void);
void Init_CGU(char MSEL, char DF, char CSEL, char SSEL);

//	ADCC Functions
void Init_AFE(void);
void Init_ADCC_Regs(void);
void Enable_ADCC(void);
void Init_ADCC_Timer_trigger(void);

//	LCD_Disply functions
void Init_TWI0(void);
void Init_Disaply(void);
void Write_Disaply(void);
void Display_ADC_para(void);
void Write_TWI_LCD_Command(unsigned char CHIP_ID, unsigned char cmd, unsigned char cmd_value, unsigned char cmd_size);
void Write_TWI_LCD_Data(unsigned char CHIP_ID, unsigned char* Tx_Buff, unsigned char count);

//	Interrupt functions
void Init_Interrupts(void);
void ADCC0_ERR_Int_Handler (void);

//	SPI Memory Mapped mode
void SetupSPI_MMode(void);
void SetupCache(void);

//	SNR Calculation functions
void calculate_ADC_Performance(void);

//	Misc
void Init_MDMA(void);
void Init_LEDs(void);
void Display_ADC_para_Terminal_IO(void);
void Clear_process_counters(void);
/***********************************************************************************************/
