/*******************************************************************
*	Function name:  main.c
*	Description	:	The SNR calculation fucntion in the Code2_Single_channe_2631 and Code2_Single_channe_2631_MDMA codes 
*					needs following co-efficients for processing:
*						-	64K floaintg point Blackman window co-effients
*						-	32K floaintg point co-sine co-efficients for 64K point FFT
*						-	32K floaintg point sine co-efficients for 64K point FFT
*					This code must be run before running those projects. This project calculates and then stores those co-effients.
*					The filter co-efficient needed for Blackman window are stored at sector 24, 25, 26 and 27
*					The co-sine co-efficients needed for FFT opeation are stored at sector 28 and 29
*					The sine co-efficients needed for FFT opeation are stored at sector 31 and 32
*	Arguments	:	None. 
*   Return value:   None.
*	Note		:	uncomemnt one of the macro defined below, rebuild the project and run the code. 
*					Then uncomment anoter macro, comment previous, rebuild the project and run the code. 
*					similarly uncomment remanining macro, comment previous, rebuild the project and run the code.
*					SO, this code should be run trice with one of the macro. Each run will programs respective co-efficients
*******************************************************************/

#include "main.h"

/*******************************************************************
*  macro defines
*******************************************************************/
//	uncomment one by one and comment others
//#define 	Load_Blackman_Coeff
//#define	Load_Cos_coeff
#define	Load_Sine_Coeff


/*******************************************************************
*  data defination 
*******************************************************************/

#ifdef Load_Blackman_Coeff
	#pragma location = ".SMC_bank0"
	float Blackman_Coeff[0x10000];
#else
	#ifdef Load_Cos_coeff
		#pragma location = ".SMC_bank0"
		float costab1q[0x4001];
		float costab[0x8000]; 
	#else
		#pragma location = ".SMC_bank0"
		float costab1q[0x4001];
		float sintab[0x8000];
	#endif
#endif

unsigned char result;


/****************************************************************************************************************************************
*	Function name:  main
*	Description	:	This main function calls the different function which either writes & verifies the Blackman window 
*					co-efficients, sine co-efficients or cosine co-efficients of FFT based on macro defination above
*	Arguments	:	None. 
*   Return value:   None.
****************************************************************************************************************************************/

void main(void)
{
	Init_CGU(32, 1, 2, 5);
	Init_SPI2();
	Init_BUFF();
	
#ifdef Load_Blackman_Coeff
	Erase_flash(24); Erase_flash(25); Erase_flash(26); Erase_flash(27);
	Write_flash(24, (unsigned char*)Blackman_Coeff, sizeof(Blackman_Coeff));
#else
	#ifdef Load_Cos_coeff
		Erase_flash(28); Erase_flash(29); 
		Write_flash(28, (unsigned char*)costab, sizeof(costab));
	#else
		Erase_flash(30); Erase_flash(31);
		Write_flash(30, (unsigned char*)sintab, sizeof(sintab));
	#endif
#endif

	while(1)
	  asm("nop;");
}


/****************************************************************************************************************************************
*	Function name:  Init_BUFF
*	Description	:	This function calculates the filter co-effieicient for blackman windowing
*					or FFT sine or cosine co-efficients based on the macro definations above. 
*	Arguments	:	None. 
*   Return value:   None.
****************************************************************************************************************************************/

void Init_BUFF(void)
{
#ifdef Load_Blackman_Coeff
	int i0;
	Blackman_Coeff[0] = 0.0;
	for(i0=1; i0 < 65536; i0++) 
		Blackman_Coeff[i0] = 0.42 - 0.5 * cos(2.0*PI_VALUE*((float) i0 / 65535.0)) + 0.08 * cos(4.0*PI_VALUE*((float) i0 / 65535.0));
#else
	int k;
	costab1q[0] = 1.0;
	for (k = 0; k < 8192; k++)
		costab1q[k + 1] = cos(9.5873799242852573E-5 * ((float)k + 1.0));

	for (k = 0; k < 8191; k++)
		costab1q[k + 8193] = sin(9.5873799242852573E-5 * (16384.0 - ((float)k + 8193.0)));

	#ifdef Load_Cos_coeff
		costab1q[16384] = 0.0;
		costab[0] = 1.0;
		for (k = 0; k < 16384; k++)
			costab[k + 1] = costab1q[k + 1];

		for (k = 0; k < 16384; k++)
			costab[k + 16385] = -costab1q[16383 - k];
	#endif
	
	#ifdef Load_Sine_Coeff
		costab1q[16384] = 0.0;
		sintab[0] = 0.0;
		for (k = 0; k < 16384; k++)
			sintab[k + 1] = -costab1q[16383 - k];

		for (k = 0; k < 16384; k++)
			sintab[k + 16385] = -costab1q[k + 1];
	#endif
#endif
}


/****************************************************************************************************************************************
*	Function name:  Init_SPI2
*	Description	:	Initialises SPI controller which is connected to internal SPI flash memory. 
*	Arguments	:	None. 
*   Return value:   None.
****************************************************************************************************************************************/

void Init_SPI2(void)
{
	*pREG_SPI2_CLK	 = 1;
	*pREG_SPI2_DLY	 = 0;	//((1 << BITP_SPI_DLY_STOP) & BITM_SPI_DLY_STOP) ;
	*pREG_SPI2_SLVSEL= ENUM_SPI_SLVSEL_SSEL1_HI | ENUM_SPI_SLVSEL_SSEL1_EN;

	*pREG_SPI2_CTL 	 = ENUM_SPI_CTL_MASTER ;
	*pREG_SPI2_TXCTL = ENUM_SPI_TXCTL_TTI_EN | ENUM_SPI_TXCTL_TWC_EN | ENUM_SPI_TXCTL_ZERO ;
	*pREG_SPI2_RXCTL = ENUM_SPI_RXCTL_OVERWRITE ;

	*pREG_SPI2_TXCTL |= ENUM_SPI_TXCTL_TX_EN;	//	enable Tx SPI
	*pREG_SPI2_RXCTL |= ENUM_SPI_RXCTL_RX_EN;	//	enable Rx SPI
	*pREG_SPI2_CTL 	 |= ENUM_SPI_CTL_EN;
}
/*****************************************************************************************************************************************************************/


unsigned char FLASH_Byte_access(unsigned char Databyte)
{
	//	read RFIFO till it is empty
	while(!(*pREG_SPI2_STAT & BITM_SPI_STAT_RFE))
		*pREG_SPI2_RFIFO;

	*pREG_SPI2_TWC	= 	1;						//	single byte instruction, no addr/data
	*pREG_SPI2_TFIFO = Databyte;						//	command ID
	while(!(*pREG_SPI2_STAT & BITM_SPI_STAT_TF));				//	wait till completion
	*pREG_SPI2_STAT = BITM_SPI_STAT_TF;					//	clear latch
	return *pREG_SPI2_RFIFO;
}
/*****************************************************************************************************************************************************************/


void FLASH_Verify_codes(void)
{
	unsigned char  MAN_ID;
	unsigned short DEV_ID;

	Select_Flash();	
	FLASH_Byte_access(0xFF);
	Deselect_Flash();
	
	Select_Flash();	
	FLASH_Byte_access(FLASH_RD_ID);					//	send command for reading flash Identification codes
	MAN_ID  = (FLASH_Byte_access(DUMMY_BYTE) );			//	Manufacturer ID
	DEV_ID  = (FLASH_Byte_access(DUMMY_BYTE) << 8);			//	Memory type ID15-8
	DEV_ID |= (FLASH_Byte_access(DUMMY_BYTE) );			//	capacity ID7-0

	Deselect_Flash();
      
	if((MAN_ID == FLASH_MAN_ID) && (DEV_ID == FLASH_DEV_ID))
		printf("\n Flash Identification passed:\n\tManufactarer ID = 0x%04x\tDevice ID = 0x%04x\n", MAN_ID, DEV_ID);
	else	{
		printf("\nERROR: Flash Identification failed\n");
		while(1) asm("nop;");	}
     
}
/*****************************************************************************************************************************************************************/


void Erase_flash(unsigned char Block_ID)
{
	printf("\nErasing the Block: %d\n", Block_ID);
	
	unsigned int Erase_ADDR = Block_ID * FLASH_BLOCK_SIZE;

	Select_Flash( );	
	FLASH_Byte_access(FLASH_WR_EN);						//	send write enable command
	Deselect_Flash( );	
	FLASH_wait_for_status(WEL, 1);						//	check whether WEN bit set properly

	Select_Flash( ) ;
	FLASH_Byte_access(FLASH_SECT_ERASES);					//	send sector erase command (0x64KB)
	FLASH_Byte_access((Erase_ADDR >> 16));					//	send block address
	FLASH_Byte_access((Erase_ADDR >> 8));
	FLASH_Byte_access(Erase_ADDR);
	Deselect_Flash( ) ;	

	FLASH_wait_for_status(WIP, 0);

	printf("Erasing Completed\n ");
	printf("\n-------------------------------------------------------------\n");
}
/*****************************************************************************************************************************************************************/


void Write_flash(unsigned char Block_ID, unsigned char* flash_page_data_buff, unsigned long long data_count)
{
	unsigned char* Flash_Read_ADDR = flash_page_data_buff;
	check_block(Block_ID);

	unsigned int PRGM_ADDR = Block_ID * FLASH_BLOCK_SIZE;
	printf("\nPogramming the Flash from Block %d:\n", Block_ID);
	while(data_count >= FLASH_PAGE_SIZE)
	{
		Write_flash_page(PRGM_ADDR, flash_page_data_buff, FLASH_PAGE_SIZE);
//		printf("\tSUCCESS: Flash Page %d programmed\n", PRGM_ADDR>>8);
		data_count	-= FLASH_PAGE_SIZE;
		flash_page_data_buff	+= FLASH_PAGE_SIZE;
		PRGM_ADDR	+= FLASH_PAGE_SIZE;
	}

	if(data_count != 0x00)
	{
		Write_flash_page(PRGM_ADDR, flash_page_data_buff, data_count);
//		printf("\tSUCCESS: Flash Page %d programmed\n", PRGM_ADDR>>8);
		flash_page_data_buff	+= data_count;
		PRGM_ADDR	+= data_count;		
	}
	printf("Flash Programming Completed\n ");
	printf("\n-------------------------------------------------------------\n");
	
	result = Compare_Flash_Data(Block_ID, Flash_Read_ADDR, data_count);
	
	if(result)
			printf("\nSUCCESS: DATA written and verified to flash\n");
	else{
			printf("\nERROR: DATA writing or verification unsuccessful\n");
			while(1)
			  asm("nop;");
		}
}

void Write_flash_page(unsigned int PAGE_ADDR, unsigned char* flash_page_data_buff, unsigned short page_data_count)
{
	Select_Flash() ;			//	selct_flash;
	FLASH_Byte_access(FLASH_WR_EN);						//	send write enable command
	Deselect_Flash() ;				//	deselct_flash;
	FLASH_wait_for_status(WEL, 1);


	Select_Flash() ;			//	selct_flash;
	FLASH_Byte_access(FLASH_PAGE_PROGRAM);					//	send page program command
	FLASH_Byte_access((PAGE_ADDR >> 16));
	FLASH_Byte_access((PAGE_ADDR >> 8));
	FLASH_Byte_access(PAGE_ADDR);

	int i;
	for(i=0; i<page_data_count; i++)
		FLASH_Byte_access(*flash_page_data_buff++);

	Deselect_Flash() ;				//	deselct_flash;

	FLASH_wait_for_status(WIP, 0);
}
/*****************************************************************************************************************************************************************/


unsigned char Compare_Flash_Data(unsigned char Block_ID, unsigned char* WR_BUFF, unsigned int data_count)
{
	check_block(Block_ID);

	unsigned int READ_ADDR = Block_ID * FLASH_BLOCK_SIZE;
	printf("\nReading the Flash from Block %d:\n", Block_ID);
	
	Select_Flash() ;				//	selct_flash;
	FLASH_Byte_access(FLASH_NORMAL_RD);					//	send sector erase command (0x64KB)
	FLASH_Byte_access((READ_ADDR >> 16));					//	send block address
	FLASH_Byte_access((READ_ADDR >> 8));
	FLASH_Byte_access(READ_ADDR);

	unsigned long long RD_count;
	unsigned char RD_Data;
	for(RD_count=0; RD_count<data_count; RD_count++, WR_BUFF++)
	{
		RD_Data = (FLASH_Byte_access(DUMMY_BYTE));
		if(RD_Data != *WR_BUFF)
		  return 0;
	}

	Deselect_Flash() ;
	printf("Flash Reading Completed\n");
	printf("\n-------------------------------------------------------------\n");
	return 1;
}
/*****************************************************************************************************************************************************************/


void FLASH_wait_for_status(unsigned char flag, unsigned char state)
{
	unsigned char flash_status;
	Select_Flash() ;			//	selct_flash;

	if(state)  {
		do {
			flash_status = FLASH_Read_STAT_Reg(FLASH_STAT_RD);
		   } while(!(flash_status & flag));		}
	else	   {
		do {
			flash_status = FLASH_Read_STAT_Reg(FLASH_STAT_RD);
		   } while(flash_status & flag);		}

	Deselect_Flash() ;				//	deselct_flash;
}


unsigned char FLASH_Read_STAT_Reg(unsigned char REG_ID)
{
  unsigned char STAT_REG;

	  	Select_Flash() ;			//	selct_flash;
  		FLASH_Byte_access(REG_ID);
  		STAT_REG = FLASH_Byte_access(REG_ID);
  		Deselect_Flash() ;				//	deselct_flash;	
		
		return STAT_REG;
}
/*****************************************************************************************************************************************************************/


void check_block(unsigned char Block_count)
{
	if(Block_count > FLASH_BLOCK_COUNT-1)		
	{
		printf("\nERROR: Invalid Flash Block ID\n");
		while(1) 
		  asm("nop;");
	}
}

void Select_Flash(void)
{
	*pREG_SPI2_SLVSEL &= ~ENUM_SPI_SLVSEL_SSEL1_HI ;
}

void Deselect_Flash(void)
{
  	*pREG_SPI2_SLVSEL |= ENUM_SPI_SLVSEL_SSEL1_HI ;				//	deselct_flash;
	*pREG_SPI2_SLVSEL &= ~ENUM_SPI_SLVSEL_SSEL1_HI ;
	*pREG_SPI2_SLVSEL &= ~ENUM_SPI_SLVSEL_SSEL1_HI ;
}
/*****************************************************************************************************************************************************************/

