/***************************************************************************************************************************************************************
*	File name 	: 	SNR_calculation.C
*	Purpose		: 	This function calculates the ADC performance parameters such as SNR, SNDR, ENOB and THD on the ADC samples acquired through ADCC.
*					The 64K point FFT code is written by referring the MATLAB code library. www.matlab.com
***************************************************************************************************************************************************************/
#include "main.h"


unsigned short ADC_Data[(24*BUFF_SIZE)];

#pragma location = ".SMC_bank0"
complex_float_var 	ADC_Data_FFT[65536];

ADC_Perfomance_Metrices        ADC_para; 
int Start_pointer;

void calculate_ADC_Performance(void)
{
	unsigned char check;
	short ADC_SIGN_DATA;
	int i0, i4, i1=0, i2=0, match=0, up_cnt=0, dwn_cnt=0, iDiff, iDiff2, iLenh, ibase;
	unsigned int i3=0, n;
	float temp_re, temp_im;
	unsigned long long ADC_Data_Sum=0;
	
	float* pBlackman_Coeff;
	pBlackman_Coeff = (float*) (0x18000000 + (BLCKMAN_WINDOW_COEFF_SECT * SPI_FLASH_BLOCK_SIZE));
	
	float* pCOS_Coeff;
	pCOS_Coeff = (float*) (0x18000000 + (COS_COEFF_SECT * SPI_FLASH_BLOCK_SIZE));
	
	float* pSIN_Coeff;
	pSIN_Coeff = (float*) (0x18000000 + (SIN_COEFF_SECT * SPI_FLASH_BLOCK_SIZE));
/*
        int p;
        for(p = 0; p<65536; p++)
          ADC_Data_FFT[p].re = *pBlackman_Coeff++;
        
        for(p = 0; p<32768; p++)
            ADC_Data_FFT[p].re = *(pCOS_Coeff+p);
        
        for(p = 0; p<32768; p++)
            ADC_Data_FFT[p].re = *(pSIN_Coeff+p);
	*/
	for(i0=0; i0<2232; i0++)
	{
		if(((ADC_Data[(sizeof(ADC_Data)/4)+i0]/100) == 326) || ((ADC_Data[(sizeof(ADC_Data)/4)+i0]/100) == 327) || ((ADC_Data[(sizeof(ADC_Data)/4)+i0]/100) == 328))
		{
			match = 1;
			up_cnt = 1;
			break;
		}
		if(((ADC_Data[(sizeof(ADC_Data)/4)-i0]/100) == 326) || ((ADC_Data[(sizeof(ADC_Data)/4)-i0]/100) == 327) || ((ADC_Data[(sizeof(ADC_Data)/4)-i0]/100) == 328))
		{
			match = 1;
			dwn_cnt = 1;
			break;
		}
	}
	
	if(match == 1)
	{
		if(up_cnt == 1)
			Start_pointer = ((sizeof(ADC_Data)/4)+i0) - 32768;
		else if (dwn_cnt == 1)
			Start_pointer = ((sizeof(ADC_Data)/4)-i0) - 32768;
	}
	else	//	Error
	{
		while(1)
			asm("nop;");
	}
	
	for(i0=0; i0<65536; i0++)
		ADC_Data_Sum += ADC_Data[Start_pointer+i0];
	ADC_Data_Sum = ADC_Data_Sum / 65536;

	for (i0 = 0, i2 = 0; i0 < 65535; i0++)
	{
		ADC_SIGN_DATA = ADC_Data[Start_pointer+i0] - ADC_Data_Sum;
		ADC_Data_FFT[i2].re = ADC_SIGN_DATA * (*(pBlackman_Coeff+i0));
		ADC_Data_FFT[i2].im = 0.0;
		n = 65536;
		check = true;
		while (check)
		{
			n >>= 1;
			i3 ^= n;
			check = ((int)(i3 & n) == 0);
		}
		i2 = (int)i3;
	}

	ADC_SIGN_DATA = ADC_Data[Start_pointer+i0] - ADC_Data_Sum;
	ADC_Data_FFT[i2].re = ADC_SIGN_DATA * (*pBlackman_Coeff++);
	ADC_Data_FFT[i2].im = 0.0;

	for (i0 = 0; i0 < 65536; i0 += 2)
	{
		temp_re = ADC_Data_FFT[i0 + 1].re;
		temp_im = ADC_Data_FFT[i0 + 1].im;
		ADC_Data_FFT[i0 + 1].re = ADC_Data_FFT[i0].re - ADC_Data_FFT[i0 + 1].re;
		ADC_Data_FFT[i0 + 1].im = ADC_Data_FFT[i0].im - ADC_Data_FFT[i0 + 1].im;
		ADC_Data_FFT[i0].re += temp_re;
		ADC_Data_FFT[i0].im += temp_im;
	}

	iDiff = 2;	iDiff2 = 4;
	i4 = 16384;	iLenh = 65533;
	while (i4 > 0)
	{
		for (i0 = 0; i0 < iLenh; i0 += iDiff2)
		{
			i1 = i0 + iDiff;
			temp_re = ADC_Data_FFT[i1].re;
			temp_im = ADC_Data_FFT[i1].im;
			ADC_Data_FFT[i0 + iDiff].re = ADC_Data_FFT[i0].re - ADC_Data_FFT[i1].re;
			ADC_Data_FFT[i0 + iDiff].im = ADC_Data_FFT[i0].im - ADC_Data_FFT[i1].im;
			ADC_Data_FFT[i0].re += temp_re;
			ADC_Data_FFT[i0].im += temp_im;
		}

		i1 = 1;
		for (i2 = i4; i2 < 32768; i2 += i4)
		{
			i0 = i1;
			ibase = i1 + iLenh;
			while (i0 < ibase)
			{
				temp_re = (*(pCOS_Coeff+i2) * ADC_Data_FFT[i0 + iDiff].re) - (*(pSIN_Coeff+i2) * ADC_Data_FFT[i0 + iDiff].im);
				temp_im = (*(pCOS_Coeff+i2) * ADC_Data_FFT[i0 + iDiff].im) + (*(pSIN_Coeff+i2) * ADC_Data_FFT[i0 + iDiff].re);
				ADC_Data_FFT[i0 + iDiff].re = ADC_Data_FFT[i0].re - temp_re;
				ADC_Data_FFT[i0 + iDiff].im = ADC_Data_FFT[i0].im - temp_im;
				ADC_Data_FFT[i0].re += temp_re;
				ADC_Data_FFT[i0].im += temp_im;
				i0 += iDiff2;
			}
			i1++;
		}

		i1 = i4;
		i4 = (int)((unsigned int)i1 >> 1);
		iDiff = iDiff2;
		iDiff2 <<= 1;
		iLenh -= iDiff;
	}

	float max_FFT = 0, noise_FFT = 0, Harmonic_dist=0;
	for(i0=0; i0<32768; i0++)
	{
		temp_re = (ADC_Data_FFT[i0].re * ADC_Data_FFT[i0].re) ;
		temp_im = (ADC_Data_FFT[i0].im * ADC_Data_FFT[i0].im) ;
		ADC_Data_FFT[i0].re = temp_re + temp_im;
	}
    
    int max_FFT_ID=0;
    for(i0=15; i0<32768; i0++)
	{
		if(ADC_Data_FFT[i0].re > max_FFT)
		{
			max_FFT_ID = i0;
			max_FFT = ADC_Data_FFT[i0].re;
		}
	}

	for(i0=1; i0<10; i0++)
	{
		max_FFT += ADC_Data_FFT[max_FFT_ID + i0].re;
		max_FFT += ADC_Data_FFT[max_FFT_ID - i0].re;
	}

	for(i0=10; i0<(max_FFT_ID - 10); i0++)
		noise_FFT += ADC_Data_FFT[i0].re;

	for(i0=(max_FFT_ID + 10); i0<32768; i0++)
		noise_FFT += ADC_Data_FFT[i0].re;

    float SNDR_DB = max_FFT/noise_FFT;
	ADC_para.SNDR = 10*log10(SNDR_DB);

	ADC_para.ENOB = (ADC_para.SNDR-1.76)/6.0205;

	for(i0=2; i0<((32768/(max_FFT_ID+1))-1); i0++)
	{
		Harmonic_dist += ADC_Data_FFT[(i0*(max_FFT_ID+1))-4].re;
		Harmonic_dist += ADC_Data_FFT[(i0*(max_FFT_ID+1))-3].re;
		Harmonic_dist += ADC_Data_FFT[(i0*(max_FFT_ID+1))-2].re;
		Harmonic_dist += ADC_Data_FFT[(i0*(max_FFT_ID+1))-1].re;
		Harmonic_dist += ADC_Data_FFT[(i0*(max_FFT_ID+1))+0].re;
		Harmonic_dist += ADC_Data_FFT[(i0*(max_FFT_ID+1))+1].re;
		Harmonic_dist += ADC_Data_FFT[(i0*(max_FFT_ID+1))+2].re;
	}

	float Harmonic_dist_noise = noise_FFT - Harmonic_dist;

	ADC_para.SNR = 10*log10(max_FFT/Harmonic_dist_noise);
	ADC_para.THD = 10*log10(Harmonic_dist/max_FFT);
}
/****************************************************************************************************************************************/

